//==========================================================================================================================
// AVR I2S audio output demo - USART functions
// Version V1.00, 10.2.2012
// 
// (C) 2012, Stanislav Maslan, s.maslan@seznam.cz
// www: http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/tutorial/AudioTest/I2Semulation/I2S_emulation.html
//==========================================================================================================================


#include <avr/io.h>
#include <avr/pgmspace.h>
#include "string.h"

#include "serial.h"
#include "main.h"
#include "sound.h"


// ------ flush rx buffer ------
inline void rx_flush(void)
{
	UDR0;
	UDR0;
	UDR0;
}



// ------ read data block into FIFO buffer ------
int8_t rx_riff(void)
{
	// init timeout
	to_time(500);

	// data destination
	FIFOwr=FIFO+RX_RIFF_OFS;

	// loose trash
	rx_flush();

	// CTS
	rx_cts_set();

	// wait for first byte
	loop_until_bit_is_set(UCSR0A,RXC0);

	// start timeout timer
	to_run();

	// recieve data
	uint8_t *wr=FIFOwr;
	while(1)
	{
		// store byte
		*wr++=UDR0;

		// check for buffer overflow
		if(wr>=FIFO+FIFOsize)
		{
			return(2);
		}

		// reset timeout
		to_reset();
		
		// check for data amount
		if(wr==FIFO+RX_RIFF_OFS+RX_RIFF_N)
		{
			// data recieved, stop reciever
			rx_cts_clr();
			
			// reset timeout to 50ms
			to_time(50);
		}

		do{
			if(to_test())
			{
				// timeout, stop timer
				to_stop();
				FIFOwr=wr;
				return(wr<FIFO+RX_RIFF_OFS+RX_RIFF_N);
			}				
		}while(bit_is_clear(UCSR0A,RXC0));		
	}	
}




// ------ recieve while byte interval is less than 500ms ------
void rx_discard(void)
{
	// CTS
	rx_cts_set();
	
	// config timeout
	to_time(500);
	to_run();
	
	while(1)
	{
		// reset timeout
		to_reset();

		// wait for data
		do{
			if(to_test())
			{
				// timeout stop
				to_stop();
				return;
			}
		}while(bit_is_clear(UCSR0A,RXC0));
		UDR0;
	}	
}





// ------ tx FLASH string ------
void tx_str_P(PGM_P str)
{
	while(pgm_read_byte(str))
	{
		loop_until_bit_is_set(UCSR0A,UDRE0);
		UDR0=pgm_read_byte(str++);
	}
}





// ------ tx string ------
void tx_str(char *str)
{
	while(*str)
	{
		loop_until_bit_is_set(UCSR0A,UDRE0);
		UDR0=*str++;
	}
}
